'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = restartable;

var _rx = require('rx');

function _typeof(obj) { return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj; }

function disposeAllStreams(streams) {
  Object.keys(streams).forEach(function (key) {
    var value = streams[key];

    delete streams[key];

    value.dispose();
  });
}

function makeDispose(_ref, originalDispose) {
  var streams = _ref.streams;

  return function dispose() {
    originalDispose();
    disposeAllStreams(streams);
  };
}

function onDispose(observable, disposeHandler) {
  var oldDispose = observable.dispose.bind(observable);

  observable.dispose = function () {
    disposeHandler();
    oldDispose();
  };

  return observable;
}

function record(_ref2, streamToRecord, identifier) {
  var streams = _ref2.streams;
  var addLogEntry = _ref2.addLogEntry;

  if (streams[identifier] === undefined) {
    streams[identifier] = new _rx.Subject();
  }

  var stream = streams[identifier];

  var subscription = streamToRecord.subscribe(function (event) {
    addLogEntry({ event: event, time: new Date(), identifier: identifier, stream: stream });

    stream.onNext(event);
  });

  onDispose(stream, function () {
    return subscription.dispose();
  });

  return stream;
}

function recordObservableSource(_ref3, source) {
  var streams = _ref3.streams;
  var addLogEntry = _ref3.addLogEntry;

  var source$ = new _rx.ReplaySubject(1);

  streams[':root'] = source$;

  var subscription = source.subscribe(function (event) {
    if (typeof event.subscribe === 'function') {
      var loggedEvent$ = event.do(function (response) {
        addLogEntry({
          event: _extends(_rx.Observable.just(response), event),
          time: new Date(),
          identifier: ':root'
        });
      });

      loggedEvent$.request = event.request;

      source$.onNext(loggedEvent$);
    } else {
      source$.onNext(event);

      addLogEntry({ event: event, time: new Date(), identifier: ':root' });
    }
  });

  onDispose(source$, function () {
    return subscription.dispose();
  });

  return source$;
}

function wrapSourceFunction(_ref4, name, f, context) {
  var streams = _ref4.streams;
  var addLogEntry = _ref4.addLogEntry;
  var scope = arguments.length <= 4 || arguments[4] === undefined ? [] : arguments[4];

  return function newSource() {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var newScope = scope.concat(args);

    var returnValue = f.bind.apply(f, [context].concat(args))();

    if (name.indexOf('isolate') !== -1 || (typeof returnValue === 'undefined' ? 'undefined' : _typeof(returnValue)) !== 'object') {
      return returnValue;
    }

    if (typeof returnValue.subscribe !== 'function') {
      return wrapSource({ streams: streams, addLogEntry: addLogEntry }, returnValue, newScope);
    }

    var identifier = newScope.join('/');

    return record({ streams: streams, addLogEntry: addLogEntry }, returnValue, identifier);
  };
}

function wrapSource(_ref5, source) {
  var streams = _ref5.streams;
  var addLogEntry = _ref5.addLogEntry;
  var scope = arguments.length <= 2 || arguments[2] === undefined ? [] : arguments[2];

  var returnValue = {};

  Object.keys(source).forEach(function (key) {
    var value = source[key];

    if (key === 'dispose') {
      returnValue[key] = makeDispose({ streams: streams }, value);
    } else if (typeof value === 'function') {
      returnValue[key] = wrapSourceFunction({ streams: streams, addLogEntry: addLogEntry }, key, value, returnValue, scope);
    } else {
      returnValue[key] = value;
    }
  });

  return returnValue;
}

function restartable(driver) {
  var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var logEntry$ = new _rx.Subject();
  var log$ = logEntry$.startWith([]).scan(function (log, entry) {
    return log.concat([entry]);
  }).shareReplay(1);

  function addLogEntry(entry) {
    logEntry$.onNext(entry);
  }

  // TODO - dispose log subscription
  log$.subscribe();

  var streams = {};

  var pauseSinksWhileReplaying = opts.pauseSinksWhileReplaying === undefined ? true : opts.pauseSinksWhileReplaying;

  var replaying = undefined;

  function restartableDriver(sink$) {
    var filteredSink$ = sink$;

    if (sink$ && pauseSinksWhileReplaying) {
      filteredSink$ = sink$.filter(function () {
        return !replaying;
      });
    }

    var source = driver(filteredSink$);

    var returnValue = undefined;

    if (source === undefined || source === null) {
      return source;
    } else if (typeof source.subscribe === 'function') {
      returnValue = recordObservableSource({ streams: streams, addLogEntry: addLogEntry }, source);
    } else {
      returnValue = wrapSource({ streams: streams, addLogEntry: addLogEntry }, source);
    }

    var oldReturnValueDispose = returnValue.dispose;

    returnValue.dispose = function () {
      oldReturnValueDispose && oldReturnValueDispose();
      sink$ && sink$.dispose();

      disposeAllStreams(streams);
    };

    returnValue.log$ = log$;

    return returnValue;
  }

  function replayable(driver) {
    driver.onPreReplay = function () {
      replaying = true;
    };

    driver.replayLog = function (scheduler, newLog$) {
      var timeToResetTo = arguments.length <= 2 || arguments[2] === undefined ? null : arguments[2];

      newLog$.take(1).subscribe(function (newLog) {
        function scheduleEvent(historicEvent) {
          scheduler.scheduleAbsolute({}, historicEvent.time, function () {
            if (streams[historicEvent.identifier]) {
              streams[historicEvent.identifier].onNext(historicEvent.event);
            } else {
              console.error('Missing replay stream ', historicEvent.identifier);
            }
          });
        }

        newLog.filter(function (event) {
          return timeToResetTo === null || event.time <= timeToResetTo;
        }).forEach(scheduleEvent);
      });
    };

    driver.onPostReplay = function () {
      replaying = false;
    };

    return restartableDriver;
  }

  return replayable(restartableDriver);
}